@@ -9,7 +9,7 @@ from sys import version_info
 import warnings
 
 # These are the types of data currently available in opd.
-# They all have corresponding data loaders in data_loaders.py
+# They all have corresponding data loaders.
 # When new data loaders are added, this list should be updated.
 class DataType(str, Enum):
     ArcGIS = "ArcGIS"
@@ -18,6 +18,7 @@ class DataType(str, Enum):
     CSV = "CSV"
     EXCEL = "Excel"
     HTML = 'HTML'
+    OPENDATASOFT = 'Opendatasoft'
     SOCRATA = "Socrata"
     
 
@@ -103,7 +104,7 @@ class TableType(str, Enum):
                           "Incidents data may be split into several tables due to the possibility that multiple "+
                         "subjects may be involved in an incident. This table contains data on subjects.")
     LAWSUITS = ("LAWSUITS", "Lawsuits against a police department")
-    PEDESTRIAN = ("PEDESTRIAN STOPS","Stops of pedestrians based on 'reasonable suspicion'. May lead to a frisk.")
+    PEDESTRIAN_STOPS = ("PEDESTRIAN STOPS","Stops of pedestrians based on 'reasonable suspicion'. May lead to a frisk.")
     PEDESTRIAN_ARRESTS = ("PEDESTRIAN ARRESTS","Pedestrian stops leading to an arrest")
     PEDESTRIAN_CITATIONS = ("PEDESTRIAN CITATIONS","Pedestrian stops leading to a citation")
     PEDESTRIAN_WARNINGS = ("PEDESTRIAN WARNINGS","Pedestrian stops leading to a warning")
@@ -138,6 +139,8 @@ class TableType(str, Enum):
     TRAFFIC_CITATIONS = ("TRAFFIC CITATIONS","Traffic stops leading to a citation.")
     TRAFFIC_WARNINGS = ("TRAFFIC WARNINGS","Traffic stops leading to a warning.")
     USE_OF_FORCE = ("USE OF FORCE","Documentation of physical force used against subjects.")
+    USE_OF_FORCE_ADDITIONAL = ("USE OF FORCE - ADDITIONAL",
+        "Table with additional use of force information")
     USE_OF_FORCE_SUBJECTS = ("USE OF FORCE - SUBJECTS",
         "Use of force data may be split into several tables due to the possibility that multiple "+
         "subjects and officers may be involved in an incident. This table contains data on subjects.")
