@@ -10,6 +10,7 @@ from openpolicedata.exceptions import OPD_DataUnavailableError, OPD_TooManyReque
 	OPD_MultipleErrors, OPD_arcgisAuthInfoError, OPD_SocrataHTTPError, OPD_FutureError, OPD_MinVersionError
 import openpolicedata as opd
 from datetime import datetime
+from io import StringIO
 import pandas as pd
 from time import sleep
 import warnings
@@ -42,15 +43,57 @@ else:
 
 warn_errors = (OPD_DataUnavailableError, OPD_SocrataHTTPError, OPD_FutureError, OPD_MinVersionError)
 
+@pytest.fixture()
+def log_stream():
+    stream = StringIO()
+    yield stream
+    stream.truncate(0)
+    stream.seek(0)
+    assert len(stream.getvalue()) == 0
+
+@pytest.fixture()
+def logger(log_stream):
+    logger = opd.log.get_logger()
+    # Redirect handler output so that it can be checked
+    logger.handlers[0].setStream(log_stream)
+
+    yield logger
+    for handler in logger.handlers:
+        if handler.name != opd.log.stream_handler_name:
+            logger.removeHandler(handler)
+
 def check_table_type_warning(all_datasets):
 	sources = all_datasets.copy().iloc[0]
 	sources["TableType"] = "TEST"
 	with pytest.warns(UserWarning):
 		data.Table(sources)
 
+
+def test_source_bad_source_name(all_datasets):
+	source_name_partial = "Charlotte"
+	with pytest.raises(ValueError, match='No sources found'):
+		opd.Source(source_name_partial)
+
+
+def test_not_verbose(logger, log_stream):
+	source = 'Lansing'
+	table = "OFFICER-INVOLVED SHOOTINGS"
+	if check_for_dataset(source, table):
+		src = opd.Source(source)
+		table = src.load(table, 'MULTIPLE')
+		assert len(log_stream.getvalue()) == 0
+
+def test_verbose(logger, log_stream):
+	source = 'Lansing'
+	table = "OFFICER-INVOLVED SHOOTINGS"
+	if check_for_dataset(source, table):
+		src = opd.Source(source)
+		table = src.load(table, 'MULTIPLE', verbose=True)
+		assert len(log_stream.getvalue())>0
+
 @pytest.mark.parametrize('source, table, year', [('Phoenix', "OFFICER-INVOLVED SHOOTINGS", 2022), 
 				('Orlando', "OFFICER-INVOLVED SHOOTINGS", 2022), ('Indianapolis', "OFFICER-INVOLVED SHOOTINGS", 2022),
-				('Philadelphia', "OFFICER-INVOLVED SHOOTINGS", 2018)])
+				('Philadelphia', "COMPLAINTS - BACKGROUND", 2018)])
 def test_format_date_false(all_datasets, source, table, year):
 	if check_for_dataset(source, table):
 		src = opd.Source(source)
@@ -156,34 +199,34 @@ def test_multiple_agency_same_source_name(all_datasets, agency):
 	if check_for_dataset('Contra Costa County', opd.defs.TableType.STOPS):
 		data.Source('Contra Costa County', agency=agency)
 
-@pytest.mark.parametrize('year, url_contains', [(2019, "APDUseOfForce/FeatureServer/0"), 
+@pytest.mark.parametrize('year, url', [(2019, "APDUseOfForce/FeatureServer/0"), 
 												([2018,2019], "APDUseOfForce/FeatureServer/0"), (2021, "APD_UseOfForce2021/FeatureServer/0")])
-def test_multiple_multiple_year_filter(all_datasets, year, url_contains):
+def test_multiple_multiple_year_filter(all_datasets, year, url):
 	if check_for_dataset('Asheville', opd.defs.TableType.USE_OF_FORCE):
 		src = data.Source('Asheville')
 		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.USE_OF_FORCE, year, None, None)
 		assert filter_by_year
 		assert dataset['Year']==opd.defs.MULTI
 		assert dataset['TableType']==opd.defs.TableType.USE_OF_FORCE
-		assert url_contains in dataset['URL']
+		assert url in dataset['URL']
 
-@pytest.mark.parametrize('year, url_contains, url_contains_test', [(2020, None, 'LMPD_STOPS_DATA_(2)'), 
+@pytest.mark.parametrize('year, url, url_test', [(2020, None, 'LMPD_STOPS_DATA_(2)'), 
 												(2021, None, 'LMPD_STOP_DATA_2021'),
 												(2021, 'LMPD_STOPS_DATA_(2)', 'LMPD_STOPS_DATA_(2)')])
-def test_overlapping_multi_and_single(all_datasets, year, url_contains, url_contains_test):
+def test_overlapping_multi_and_single(all_datasets, year, url, url_test):
 	if check_for_dataset('Louisville', opd.defs.TableType.TRAFFIC):
 		src = data.Source('Louisville')
-		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.TRAFFIC, year, url_contains, None)
-		assert url_contains_test in dataset['URL']
+		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.TRAFFIC, year, url, None)
+		assert url_test in dataset['URL']
 
-@pytest.mark.parametrize('year, id_contains, id_contains_test', [(2020, None, 'ex94-c5ad'), 
+@pytest.mark.parametrize('year, id, id_test', [(2020, None, 'ex94-c5ad'), 
 												(2021, None, 'izhu-764k'),
 												([2020,2021], 'izhu-764k', 'izhu-764k')])
-def test_overlapping_multi_and_single_with_id(all_datasets, year, id_contains, id_contains_test):
+def test_overlapping_multi_and_single_with_id(all_datasets, year, id, id_test):
 	if check_for_dataset('Mesa', opd.defs.TableType.CALLS_FOR_SERVICE):
 		src = data.Source('Mesa')
-		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.CALLS_FOR_SERVICE, year, None, id_contains)
-		assert id_contains_test == dataset['dataset_id']
+		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.CALLS_FOR_SERVICE, year, None, id)
+		assert id_test == dataset['dataset_id']
 
 @pytest.mark.parametrize('year', [2020, [2020,2021], opd.defs.MULTI])
 def test_multiple_bad_multiple_year_filter(all_datasets, year):
@@ -193,15 +236,15 @@ def test_multiple_bad_multiple_year_filter(all_datasets, year):
 			src._Source__filter_for_source(opd.defs.TableType.USE_OF_FORCE, year, None, None)
 
 @pytest.mark.parametrize('year, exp_filt_by_year', [([2021,2022], True), (opd.defs.MULTI, False)])
-def test_url_contains(all_datasets, year, exp_filt_by_year):
+def test_url_input(all_datasets, year, exp_filt_by_year):
 	if check_for_dataset('Asheville', opd.defs.TableType.USE_OF_FORCE):
 		src = data.Source('Asheville')
-		url_contains = "APDUseOfForce/FeatureServer/0"
-		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.USE_OF_FORCE, year, url_contains, None)
+		url = "APDUseOfForce/FeatureServer/0"
+		dataset, filter_by_year = src._Source__filter_for_source(opd.defs.TableType.USE_OF_FORCE, year, url, None)
 		assert filter_by_year == exp_filt_by_year
 		assert dataset['Year']==opd.defs.MULTI
 		assert dataset['TableType']==opd.defs.TableType.USE_OF_FORCE
-		assert url_contains in dataset['URL']
+		assert url in dataset['URL']
 
 
 def test_offsets_and_nrows():
@@ -271,12 +314,12 @@ def test_source_download_limitable(datasets, source, start_idx, skip, loghtml, q
 
 			# Handle cases where URL is required to disambiguate requested dataset
 			ds_filter, _ = src._Source__filter_for_source(table_type, datasets.iloc[i]["Year"], None, None, errors=False)
-			url_contains = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
-			id_contains = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+			url = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+			id = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
 
 			try:
 				table = src.load(table_type, datasets.iloc[i]["Year"], pbar=True, nrows=nrows, 
-					 url_contains=url_contains, id_contains=id_contains)
+					 url=url, id=id)
 			except warn_errors as e:
 				e.prepend(f"Iteration {i}", srcName, table_type, datasets.iloc[i]["Year"])
 				update_outages(outages_file, datasets.iloc[i], True, e)
@@ -291,10 +334,8 @@ def test_source_download_limitable(datasets, source, start_idx, skip, loghtml, q
 			except:
 				raise
 
-			if len(table.table)==0 and has_outages and \
-				(outages[["State","SourceName","Agency","TableType","Year"]] == datasets.iloc[i][["State","SourceName","Agency","TableType","Year"]]).all(axis=1).any():
-				caught_exceptions_warn.append(f'Outage continues for {str(datasets.iloc[i][["State","SourceName","Agency","TableType","Year"]])}')
-				update_outages(outages_file, datasets.iloc[i], True)
+			if len(table.table)==0: 
+				update_outages(outages_file, datasets.iloc[i], True, ValueError('Table has 0 rows'))
 				continue
 			else:
 				assert len(table.table)>0
@@ -302,7 +343,7 @@ def test_source_download_limitable(datasets, source, start_idx, skip, loghtml, q
 			update_outages(outages_file, datasets.iloc[i], False)
 
 			if pd.notnull(datasets.iloc[i]['query']):
-				for k,v in data_loaders.str2json(datasets.iloc[i]['query']).items():
+				for k,v in data_loaders.data_loader.str2json(datasets.iloc[i]['query']).items():
 					assert (table.table[k]==v).all()
 
 
@@ -346,7 +387,8 @@ def test_source_download_limitable(datasets, source, start_idx, skip, loghtml, q
 
 			if not pd.isnull(datasets.iloc[i]["date_field"]):
 				if datasets.iloc[i]["date_field"] not in table.table or table.table[datasets.iloc[i]["date_field"]].isnull().all():
-					table = src.load(table_type, datasets.iloc[i]["Year"], pbar=False, nrows=2000)
+					table = src.load(table_type, datasets.iloc[i]["Year"], pbar=False, nrows=2000,
+					 	url=url, id=id)
 				assert datasets.iloc[i]["date_field"] in table.table
 				#assuming a Pandas string dtype('O').name = object is okay too
 				assert (table.table[datasets.iloc[i]["date_field"]].dtype.name in ['datetime64[ns]', 'datetime64[ns, UTC]', 
@@ -393,6 +435,7 @@ def test_source_download_limitable(datasets, source, start_idx, skip, loghtml, q
 	(data_loaders.Arcgis, "Charlotte-Mecklenburg", 'EMPLOYEE', None, []),
 	(data_loaders.Csv, 'Denver', "OFFICER-INVOLVED SHOOTINGS", None, []),
 	(data_loaders.Excel, 'Rutland', "USE OF FORCE", None, []),
+	(data_loaders.Opendatasoft, 'Long Beach', "STOPS", None, [2021, [2020, 2022]]),
 	(data_loaders.Carto, "Philadelphia", 'STOPS', None, [2021, [2020, 2022]])
 	])
 def test_get_count(datasets, loader, source, table_type, agency, years):
@@ -400,7 +443,12 @@ def test_get_count(datasets, loader, source, table_type, agency, years):
 		print(f"Testing {loader} source")
 		src = opd.Source(source)
 		i = src.datasets["TableType"] == table_type
-		i = i[i].index[0]
+		i = i[i].index
+		if len(i)>0:
+			i = src.datasets.loc[i, 'DataType'].str.lower()==loader.__name__.lower()
+			i = i[i].index
+		assert len(i)==1
+		i = i[0]
 		if pd.notnull(src.datasets.loc[i]["dataset_id"]):
 			loader = loader(src.datasets.loc[i]["URL"], data_set=src.datasets.loc[i]["dataset_id"], date_field=src.datasets.loc[i]["date_field"])
 		else:
@@ -444,7 +492,6 @@ def test_load_gen(source, year, table_type, nrows, agency):
 
 		offset = 0
 		k = 0
-		# df_iter = pd.DataFrame(columns=df.columns)
 		for t in src.load_iter(table_type, year, nbatch=nrows, force=True, agency=agency):
 			df_cur = df.iloc[offset:offset+len(t.table)].reset_index(drop=True)
 			df2 = t.table.copy()
@@ -461,43 +508,16 @@ def test_load_gen(source, year, table_type, nrows, agency):
 			# Ensure that dtypes match
 			df2 = df2.astype(df_cur.dtypes.to_dict())
 			assert df2.equals(df_cur)
-			# df_iter = pd.concat([df_iter, df2], ignore_index=True)
 			offset+=len(t.table)
 			k+=1
 			if k>=max_iter:
 				break
 
-		# assert df_iter.shape == df.shape
-		# if not df_iter.equals(df):
-		# 	# Try to match rows
-		# 	unmatched = pd.Series(True, index=df_iter.index)
-		# 	for k in df.index:
-		# 		df_cur = df_iter[unmatched].copy()
-		# 		for c in df.columns:
-		# 			if pd.isnull(df.loc[k,c]):
-		# 				m = df_cur[c].isnull()
-		# 			else:
-		# 				m = df_cur[c]==df.loc[k,c]
-
-		# 			df_cur = df_cur[m]
-		# 			if len(df_cur)==0:
-		# 				raise ValueError(f"No matches found for index {k}")
-		# 			elif len(df_cur)==1:
-		# 				break
-
-		# 		for j in df_cur.index:
-		# 			if df_cur.loc[j].equals(df.loc[k]):
-		# 				unmatched[j] = False
-		# 				break
-		# 		else:
-		# 			raise ValueError(f"No match found for row {k}")
-
-
 
 def can_be_limited(data_type, url):
 	if url.lower().endswith(".zip"):
 		return False
-	elif data_type in [DataType.ArcGIS, DataType.SOCRATA, DataType.CSV, DataType.EXCEL, DataType.CARTO, DataType.CKAN, DataType.HTML]:
+	elif data_type in [DataType.ArcGIS, DataType.SOCRATA, DataType.CSV, DataType.EXCEL, DataType.CARTO, DataType.CKAN, DataType.HTML, DataType.OPENDATASOFT]:
 		return True
 	else:
 		raise ValueError("Unknown table type")
@@ -535,13 +555,13 @@ if __name__ == "__main__":
 	use_changed_rows = False
 	csvfile = None
 	csvfile = os.path.join(r"..",'opd-data','opd_source_table.csv')
-	start_idx = 0
+	start_idx = 890
 	skip = None
 	# skip = "Sacramento"
 	source = None
 	# source = "St. Paul" #"Washington D.C." #"Wallkill"
 	query = {}
-	query = {'DataType':'CKAN'}
+	# query = {'SourceName':'San Francisco'}
 
 	datasets = get_datasets(csvfile, use_changed_rows)
 
