@@ -9,7 +9,6 @@ from openpolicedata.defs import MULTI, DataType
 from openpolicedata.exceptions import OPD_DataUnavailableError, OPD_TooManyRequestsError,  \
 	OPD_MultipleErrors, OPD_arcgisAuthInfoError, OPD_SocrataHTTPError, OPD_FutureError, OPD_MinVersionError, \
 	DateFilterException
-import random
 from datetime import datetime
 import numpy as np
 import pandas as pd
@@ -19,7 +18,7 @@ import os
 import re
 from urllib.parse import urlparse
 
-from test_utils import check_for_dataset, user_request_skip
+from test_utils import check_for_dataset
 
 # Set Arcgis data loader to validate queries with arcgis package if installed
 data_loaders._verify_arcgis = True
@@ -172,8 +171,8 @@ def test_load_year(datasets, source, start_idx, skip, loghtml, query={}):
 
 		# Handle cases where URL is required to disambiguate requested dataset
 		ds_filter, _ = src._Source__filter_for_source(datasets.iloc[i]["TableType"], datasets.iloc[i]["Year"], None, None, errors=False)
-		url_contains = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
-		id_contains = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+		url = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+		id = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
 
 		tables = {}
 		future_error = False
@@ -183,11 +182,11 @@ def test_load_year(datasets, source, start_idx, skip, loghtml, query={}):
 								agency=agency, pbar=False, 
 								sortby="date",
 								nrows=max_count if datasets.iloc[i]["DataType"] not in ["CSV","Excel"] else None, 
-					 			url_contains=url_contains, id_contains=id_contains)
+					 			url=url, id=id)
 			except OPD_FutureError as e:
 				future_error = True
 				break
-			except (OPD_DataUnavailableError, OPD_SocrataHTTPError) as e:
+			except (OPD_DataUnavailableError, OPD_SocrataHTTPError, OPD_MinVersionError) as e:
 				caught_exceptions_warn.append(e)
 				tables[year] = None
 				continue
@@ -290,14 +289,16 @@ def test_load_year(datasets, source, start_idx, skip, loghtml, query={}):
 
 			try:
 				table_start = src.load(datasets.iloc[i]["TableType"], [start_date, stop_date], 
-												agency=agency, pbar=False)
+												agency=agency, pbar=False, url=url, id=id)
 			except ValueError as e:
 				if str(e).startswith('Year range cannot contain the year corresponding to a single year dataset'):
 					start_date  = str(year) + "-01-01"
 					table_start = src.load(datasets.iloc[i]["TableType"], [start_date, stop_date], 
-												agency=agency, pbar=False)
+												agency=agency, pbar=False, url=url, id=id)
 				else:
 					raise
+			except (OPD_DataUnavailableError, OPD_SocrataHTTPError, OPD_MinVersionError) as e:
+				continue
 			except DateFilterException as e:
 				continue
 
@@ -328,13 +329,13 @@ def test_load_year(datasets, source, start_idx, skip, loghtml, query={}):
 
 			try:
 				table_stop = src.load(datasets.iloc[i]["TableType"], [start_date, stop_date], 
-												agency=agency, pbar=False)
+												agency=agency, pbar=False, url=url, id=id)
 			except ValueError as e:
 				if str(e).startswith('There is more than one source matching') or \
 					str(e).startswith('Year range cannot contain the year corresponding to a single year dataset'):
 					stop_date  = str(year) + "-12-31"  
 					table_stop = src.load(datasets.iloc[i]["TableType"], [start_date, stop_date], 
-												agency=agency, pbar=False)
+												agency=agency, pbar=False, url=url, id=id)
 				else:
 					raise
 			sleep(sleep_time)
@@ -369,10 +370,14 @@ def test_load_year(datasets, source, start_idx, skip, loghtml, query={}):
 			warnings.warn(str(e))
 
 
-def test_source_download_not_limitable(datasets, source, start_idx, skip, query={}):
+def test_source_download_not_limitable(datasets, source, start_idx, skip, query={}, num_zips_check=5):
+
+	df_ori = pd.read_csv('https://data-openjustice.doj.ca.gov/sites/default/files/dataset/2024-07/UseofForce_ORI-Agency_Names_2023.csv')
 	
 	zip_data = datasets[datasets['URL'].str.lower().str.endswith('.zip')]
-	num_zips_check = 5  # Just check a selection of these. No reason to check all
+	zip_data = zip_data[~zip_data['SourceName'].isin(skip)] if skip!=None else zip_data
+	zip_data = zip_data[zip_data.index>=start_idx]
+	zip_data = zip_data[zip_data['SourceName']==source] if source!=None else zip_data
 	zip_netloc = zip_data['URL'].apply(lambda x: urlparse(x).netloc)
 	zip_sites = zip_netloc.value_counts()
 	zip_2_run = {}
@@ -409,13 +414,13 @@ def test_source_download_not_limitable(datasets, source, start_idx, skip, query=
 
 			# Handle cases where URL is required to disambiguate requested dataset
 			ds_filter, _ = src._Source__filter_for_source(datasets.iloc[i]["TableType"], datasets.iloc[i]["Year"], None, None, errors=False)
-			url_contains = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
-			id_contains = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+			url = datasets.iloc[i]['URL'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
+			id = datasets.iloc[i]['dataset_id'] if isinstance(ds_filter,pd.DataFrame) and len(ds_filter)>1 else None
 
 			now = datetime.now().strftime("%d.%b %Y %H:%M:%S")
 			print(f"{now} Testing {i+1} of {len(datasets)}: {srcName}, {state} {table_type} table for {year}")
 			try:
-				table = src.load(table_type, year, pbar=False, url_contains=url_contains, id_contains=id_contains)
+				table = src.load(table_type, year, pbar=False, url=url, id=id)
 			except (OPD_FutureError, OPD_MinVersionError):
 				continue
 			except:
@@ -431,6 +436,27 @@ def test_source_download_not_limitable(datasets, source, start_idx, skip, query=
 			if not pd.isnull(datasets.iloc[i]["agency_field"]):
 				assert datasets.iloc[i]["agency_field"] in table.table
 
+			if table.urls['source_url'] == 'https://openjustice.doj.ca.gov/data':
+				ori = table.table['AGENCY_ORI'].unique()
+				if table.agency==MULTI:
+					assert len(ori)>1
+				else:
+					assert len(ori)==1
+					if ori[0]=='CA0349902':
+						# This ORI is not found but is California Hwy Patrol
+						continue
+					df_sel = df_ori.loc[df_ori['ORI']==ori[0],'AGENCY_NAME']
+					assert len(df_sel)==1
+					name = df_sel.iloc[0].replace('PD','Police Department')\
+						.replace('SO','County Sheriff’s Department')\
+						.replace('SD','County Sheriff’s Department')\
+						.title()\
+						.replace("’S","’s").replace('’',"'")
+					if name.startswith('Csu'):
+						assert 'California State University' in datasets.iloc[i]['AgencyFull']
+					else:
+						assert datasets.iloc[i]['AgencyFull'].replace('Office','Department').replace('’',"'")==name
+
 
 def log_errors_to_file(*args):
 	if not os.path.exists(log_folder):
@@ -464,16 +490,16 @@ if __name__ == "__main__":
 	use_changed_rows = False
 	csvfile = None
 	csvfile = os.path.join('..','opd-data', 'opd_source_table.csv')
-	start_idx = 185
+	start_idx = 0
 	skip = None
 	# skip = "Sacramento,Beloit,Rutland"
 	source = None
-	# source = "Chicago"
+	# source = "Austin"
 	query = {}
-	# query = {'DataType':'CSV'}
+	query = {'DataType':'Socrata'}
 
 	datasets = get_datasets(csvfile, use_changed_rows)
 
 	test_load_year(datasets, source, start_idx, skip, False, query=query) 
-	start_idx = 414
+	start_idx = 0
 	# test_source_download_not_limitable(datasets, source, start_idx, skip) 
